# %%

import os
import json

import pandas as pd
from datetime import datetime,timedelta
import tlid

from jgtutils.coltypehelper import DTYPE_DEFINITIONS

#@STCGoal Standardize the Signal Columns
from jgtml.mlconstants import (
  NORMAL_MOUTH_IS_OPEN_COLNAME,
  CURRENT_BAR_IS_OUT_OF_NORMAL_MOUTH_COLNAME,
  CURRENT_BAR_IS_IN_BIG_TEETH_COLNAME,
  BIG_MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_BIG_LIPS_COLNAME,
  MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_BIG_TEETH_COLNAME,
  CURRENT_BAR_IS_IN_TIDE_TEETH_COLNAME,
  TIDE_MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_TIDE_LIPS_COLNAME,
  MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_TIDE_TEETH_COLNAME
)

# Validity of cache


def is_cache_valid(df, timeframe:str,use_utc=True,quiet=True):
  """
  Checks if the cached data is still valid for the given timeframe.

  Args:
    df: The cached DataFrame.
    timeframe: The timeframe to check.

  Returns:
    True if the cache is valid, False otherwise.
  """

  # Get the last bar's timestamp.
  if "Date" in df.columns:
    last_bar_timestamp = df.iloc[-1]["Date"]
  else:
    last_bar_timestamp = df.index[-1]
  
  # Ensure the timestamp is in UTC if required
    if use_utc:
      last_bar_timestamp = pd.to_datetime(last_bar_timestamp, utc=True)
      now = datetime.utcnow().replace(tzinfo=pd.Timestamp.utcnow().tzinfo)
    else:
      last_bar_timestamp = pd.to_datetime(last_bar_timestamp)
      now = datetime.now()
  if not quiet:
    print(f"DEBUG::UTC::last_bar_timestamp:{last_bar_timestamp} now:{now}")
  # Calculate the valid range for the timeframe.
  if timeframe == "m1":
    valid_range = pd.Timedelta(minutes=1)
  elif timeframe == "m5":
    valid_range = pd.Timedelta(minutes=5)
  elif timeframe == "m15":
    valid_range = pd.Timedelta(minutes=15)
  elif timeframe == "m30":
    valid_range = pd.Timedelta(minutes=30)
  elif timeframe == "H1":
    valid_range = pd.Timedelta(hours=1)
  elif timeframe == "H2":
    # Calculate the next expiration time for H2 timeframe
    next_expiration = last_bar_timestamp.replace(minute=0, second=0, microsecond=0)
    while next_expiration <= now:
      next_expiration += timedelta(hours=2)
    is_within_expiration = now < next_expiration
    return is_within_expiration
  elif timeframe == "H3":
    next_expiration = last_bar_timestamp.replace(minute=0, second=0, microsecond=0)
    while next_expiration <= now:
      next_expiration += timedelta(hours=3)
    is_within_expiration = now < next_expiration
    return is_within_expiration
  elif timeframe == "H4":
    # Calculate the next expiration time for H4 timeframe
    next_expiration = last_bar_timestamp.replace(minute=0, second=0, microsecond=0)
    while next_expiration <= now:
      next_expiration += timedelta(hours=4)
    is_within_expiration = now < next_expiration
    return is_within_expiration
  elif timeframe == "H6":
    next_expiration = last_bar_timestamp.replace(minute=0, second=0, microsecond=0)
    while next_expiration <= now:
      next_expiration += timedelta(hours=6)
    is_within_expiration = now < next_expiration
    return is_within_expiration
  elif timeframe == "H8":
    next_expiration = last_bar_timestamp.replace(minute=0, second=0, microsecond=0)
    while next_expiration <= now:
      next_expiration += timedelta(hours=8)
    is_within_expiration = now < next_expiration
    return is_within_expiration
  elif timeframe == "D1":
    valid_range = pd.Timedelta(days=1)
  elif timeframe == "W1":
    valid_range = pd.Timedelta(days=7)
  elif timeframe == "M1":
    valid_range = pd.Timedelta(days=30)
  else:
    raise ValueError("Invalid timeframe.")
  if not quiet:
    print("DEBUG::valid_range:",valid_range)
  # Check if the last bar's timestamp is within the valid range.
  return last_bar_timestamp + valid_range > now


# %% [markdown]
# # --@STCGoal Proto Scan FDB Signal Analysis
# 

# %%
from jgtpy import JGTCDSSvc as svc

from jgtutils.jgtconstants import LOW,HIGH,FDB

import JGTBalanceAnalyzer as ba
import pandas as pd

# %% [markdown]

# # CDS Data gets added ctx bar Ctx gator
use_cache=True
cds_cache_file_suffix = "_cds_cache"
cache_root_dir="/var/lib/jgt/cache"
#look if writable, able to create otherwise use $HOME/.cache/jgt/cache
if not os.access(cache_root_dir, os.W_OK):
    cache_root_dir=os.path.join(os.getenv("HOME","~"),".cache/jgt/cache")
    try:
      os.makedirs(cache_root_dir,exist_ok=True)
    except:
      raise Exception("Unable to create cache dir")
    print("Using HOME cache dir")
    if not os.access(cache_root_dir, os.W_OK):
        print("Cache dir not writable")
        raise Exception("Cache dir not writable")
# 
# 

# %%
def _make_cached_filepath(i, t,subdir="fdb_scanners",ext="csv",suffix=""):
  ifn=i.replace("/","-")
  fn = f"{ifn}_{t}{suffix}.{ext}"
  #make sure the subdir exists
  cache_dir_fullpath=os.path.join(cache_root_dir,subdir)
  os.makedirs(cache_dir_fullpath,exist_ok=True)
  fpath=os.path.join(cache_dir_fullpath,fn)
  return fpath.replace("..",".")

def generate_fresh_and_cache(_i,_t,_quotescount=300,cache_filepath=None):
    if cache_filepath is None:
      cache_filepath = _make_cached_filepath(_i, _t,suffix=cds_cache_file_suffix)
    dfsrc:pd.DataFrame=svc.get(_i,_t,quotescount=_quotescount)
    dfsrc.to_csv(cache_filepath)
    return dfsrc

def main():

  # %%
  # i="SPX500"
  # i="NZD/CAD"
  # t="D1"
  quotescount=333
  lots=2

  md_df_tail_amount = 50
  outdir="output"
    
  contexes_all = {
      "tide": {"title": "Tide Alligator"},
      "big": {"title": "Big Alligator", "name": "big"},
      "normal": {"title": "Normal Alligator", "name": "normal"}
  }
  contexes = {
      "tide": {"title": "Tide Alligator"},
      "big": {"title": "Big Alligator", "name": "big"}
  }

  save_bars=False

  instruments = "AUD/NZD,NZD/CAD,AUD/CAD,SPX500,EUR/USD,GBP/USD,XAU/USD,USD/CAD"
  instruments = "AUD/NZD,NZD/CAD,AUD/CAD"
  instruments = "SPX500,EUR/USD,GBP/USD,AUD/USD,XAU/USD,USD/CAD,AUS200,USD/JPY,EUR/CAD,AUD/CAD,NZD/CAD,AUD/NZD"
  instruments="SPX500"
  instruments = "SPX500,EUR/USD,GBP/USD,AUD/USD,XAU/USD,USD/CAD,USD/JPY,EUR/CAD,AUD/CAD,NZD/CAD,AUD/NZD,CAD/JPY"

  instruments=os.getenv("INSTRUMENTS",instruments)
  if os.getenv("INSTRUMENTS") is not None:
      print("INSTRUMENTS loaded from environment")
  lots = int(os.getenv("LOTS",lots))
  if os.getenv("LOTS") is not None:
      print("LOTS loaded from environment")
      
  timeframes = "H1,m15,m5,m1"
  timeframes = "D1,H4,H1,m15,m5,m1"
  timeframes = "D1,H4,H2,H1,m15,m5,m1"
  timeframes = "H8,H6,H4,H3,H2,H1,m15,m5"
  timeframes = "H8,H4,H1,m15,m5"
  timeframes = "m15,m5,m1"
  timeframes = "H4,H1,m15,m5"
  timeframes = "M1,W1,D1,H4,H1,m15,m5"
  timeframes = "D1,H4,H1,m15,m5"
  timeframes = "m15,m5,m1"
  timeframes = "D1,H4,H1,m15,m5"
  timeframes = os.getenv("TIMEFRAMES",timeframes)
  if os.getenv("TIMEFRAMES") is not None:
    print("TIMEFRAMES loaded from environment")

  previous_tlid_id = tlid.get_seconds()

      
  for i in instruments.split(","):

    for t in timeframes.split(","):
      if t == " " or t == "":
        continue
      output_string = f"# Scanning: {i} {t}"
      
      cache_filepath = _make_cached_filepath(i, t,suffix=cds_cache_file_suffix)

      try:
        dfsrc=pd.read_csv(cache_filepath,index_col=0,parse_dates=True,dtype=DTYPE_DEFINITIONS)
      except:
        dfsrc:pd.DataFrame=generate_fresh_and_cache(i,t,quotescount,cache_filepath)
    
      if not is_cache_valid(dfsrc, t):
        dfsrc=generate_fresh_and_cache(i,t,quotescount)
      
      last_bar_timestamp = dfsrc.index[-1]
      
      current_bar = dfsrc.iloc[-1]
      signal_bar = dfsrc.iloc[-2]
      if save_bars:
          current_bar_fn=_make_cached_filepath(i,t,suffix="_currentbar")
          last_bar_completed_fn=_make_cached_filepath(i,t,suffix="_signalbar")
          current_bar.to_csv(current_bar_fn,index=True) 
          signal_bar.to_csv(last_bar_completed_fn,index=True) 


      
      # %%
      tlid_id = tlid.get_seconds()
      while tlid_id == previous_tlid_id:
        tlid_id = tlid.get_seconds()
        #print(".",end="")
      
      from SignalOrderingHelper import create_fdb_entry_order
      valid_gator_mouth_open_in_mouth=False
      valid_sig_out_mouth=True
      o=create_fdb_entry_order(i,signal_bar,current_bar,lots=lots,t=t,
                                          valid_gator_mouth_open_in_mouth=valid_gator_mouth_open_in_mouth,
                                          valid_sig_out_mouth=valid_sig_out_mouth)
      if o is not None:
        sh=o["sh"]
        #output_string+="\n"
        #output_string+="\n----\n"
        _found_signal_string = f" - Signal Found"
        output_string+=_found_signal_string
        output_string+=sh
        print(output_string)
        from JGTOutputHelper import serialize_signal_to_json_file,serialize_signal_to_markdown_file_from_json_file
        signal_savepath=serialize_signal_to_json_file(i,t,o,signal_bar,current_bar)
        md_filepath=serialize_signal_to_markdown_file_from_json_file(signal_savepath)
      else:
        print(output_string)

      
      previous_tlid_id = tlid_id
      
      
      process_balancing=False


      if process_balancing:

        # %%

        # #refresh the import
        # import importlib
        # importlib.reload(ba)

        r={}
        _df=None
        for bs in ["B","S"]:
          obs={}
          print("bs:",bs)
          for ctx_name,v in contexes.items():
            o={}
            #_df=dfsrc if o["df"] is None else o["df"] # Reused the new df
            if _df is None:
              _df=dfsrc
            #_df=dfsrc if _df is None else _df
            ocols=ba.generate_column_names_for_direction(ctx_name,bs)
            ctx_evaltitle =v["title"]
            o["ctx_evaltitle"]=ctx_evaltitle
            o["name"]=ctx_name
            print("k:",ctx_name," v:",ctx_evaltitle)
            #sig_ctx_mouth_is_open_and_in_ctx_lips
            df_filter=ba.filter_sig_ctx_mouth_is_open_and_in_ctx_lips(_df,bs,ctx_name)
            new_col="sig_ctx_mouth_is_open_and_in_ctx_lips"
            _df=ba.add_sig_ctx_mouth_is_open_and_in_ctx_lips(_df,bs,ctx_name,None)
            #add_sig_ctx_mouth_is_open_and_in_ctx_lips_sell
            #print(df.tail(2))
            o["df"]=_df
            o["df_filter"]=df_filter
            obs[ctx_name]=o
          r[bs]=obs
          
          #df_sig_is_in_ctx_teeth = filter_sig_is_in_ctx_teeth_sell(df_sig_is_out_of_normal_mouth, cteeth_colname, teval_colname) if bs=="S" else filter_sig_is_in_ctx_teeth_buy(df_sig_is_out_of_normal_mouth, cteeth_colname, teval_colname)
          # df_sig_is_in_ctx_teeth = filter_sig_is_in_ctx_teeth_sell(df_sig_is_out_of_normal_mouth, cteeth_colname, teval_colname) if bs=="S" else filter_sig_is_in_ctx_teeth_buy(df_sig_is_out_of_normal_mouth, cteeth_colname, teval_colname)
          
          
            

        # %%

        os.makedirs(outdir,exist_ok=True)

        rb=r["B"]
        rs=r["S"]
        content=f"""

        """

        for ctx_name,v in contexes.items():
          ctx_evaltitle =v["title"]
          rb_ctx=rb[ctx_name]
          rs_ctx=rs[ctx_name]
          print("ctx_name:",ctx_name)
          rb_ctx_df=rb_ctx["df"]
          rs_ctx_df=rs_ctx["df"]
          #save
          df_outputfile = _make_cached_filepath(i,t,subdir=outdir,suffix=f"_bs")
          _df.to_csv(df_outputfile)
          
          #make some markdown output
          content=content+f"""

  # Total number of rows:
  {len(rb_ctx_df)}
        
  # {ctx_evaltitle} Buy
  {rb_ctx_df.tail(md_df_tail_amount).to_markdown()}

  ## Columns
  {rb_ctx_df.columns.to_list()}

  # {ctx_evaltitle} Sell
  {rs_ctx_df.tail(md_df_tail_amount).to_markdown()}

  ## Columns
  {rs_ctx_df.columns.to_list()}

          """

        #save
        md_output_filepath = _make_cached_filepath(i,t,subdir=outdir,suffix=f"_ctx",ext="md")
        with open(md_output_filepath,"w") as f:
            f.write(content)
          
          
      # # %%

      # rb_tide=rb["tide"]
      # rs_tide=rs["tide"]

      # # %%

      # rb_tide_df=rb_tide["df"]
      # rs_tide_df=rs_tide["df"]

      # # %%

      # rb_tide_df.to_csv(f"{outdir}/rb_tide_df.csv")
      # rs_tide_df.to_csv(f"{outdir}/rs_tide_df.csv")



if __name__ == "__main__":
    main()




# This file is generated by pto-pre-build.sh. Do not edit it manually.







# DONT EDIT THIS FILE MANUALLY. IT IS GENERATED BY pto-pre-build.sh








#DONT EDIT THIS FILE MANUALLY. IT IS GENERATED BY pto-pre-build.sh






# Fri 16 Aug 2024 12:49:40 AM EDT
# SOURCE NAME: /b/Dropbox/jgt/drop/fnml.py
########################
 

import argparse
import subprocess

from jgtutils.jgtcliconstants import (CLI_FXADDORDER_PROG_NAME,CLI_FXMVSTOP_PROG_NAME,CLI_FXRMORDER_PROG_NAME,CLI_FXRMTRADE_PROG_NAME,CLI_FXTR_PROG_NAME,PDSCLI_PROG_NAME)

from jgtutils.jgtconstants import (LIPS,TEETH,JAW)

from jgtpy.jgtpyconstants import (IDSCLI_PROG_NAME,CDSCLI_PROG_NAME,ADSCLI_PROG_NAME,MKSCLI_PROG_NAME,JGTCLI_PROG_NAME)

try:
  from jgtml.mlcliconstants import (MLFCLI_PROG_NAME,TTFCLI_PROG_NAME,PNCLI_PROG_NAME,MXCLI_PROG_NAME)
except:
  from mlcliconstants import (MLFCLI_PROG_NAME,TTFCLI_PROG_NAME,PNCLI_PROG_NAME,MXCLI_PROG_NAME) # type: ignore


def fxaddorder( instrument, lots, rate, buysell, stop, demo=False,flag_pips=False):
  pips_arg = '--pips' if flag_pips else ''
  demo_arg = '--demo' if demo else '--real'
  subprocess.run([CLI_FXADDORDER_PROG_NAME, '-i', instrument, '-n', lots, '-r', rate, '-d', buysell, '-x',stop,pips_arg , demo_arg])

def fxrmorder(orderid, demo=False):
  demo_arg = '--demo' if demo else '--real'
  subprocess.run([CLI_FXRMORDER_PROG_NAME, '-id', orderid, demo_arg])

def fxrmtrade(tradeid, demo=False):
  demo_arg = '--demo' if demo else '--real'
  subprocess.run([CLI_FXRMTRADE_PROG_NAME, '-tid', tradeid, demo_arg])

def fxtr(tradeid=None,orderid=None, demo=False,save_flag=True):
  save_arg = '-save' if save_flag else ''
  demo_arg = '--demo' if demo else '--real'
  if tradeid:
    subprocess.run([CLI_FXTR_PROG_NAME, '-tid', tradeid, demo_arg, save_arg])
  elif orderid:
    subprocess.run([CLI_FXTR_PROG_NAME, '-id', orderid, demo_arg, save_arg])
  else:
    subprocess.run([CLI_FXTR_PROG_NAME, demo_arg, save_arg])

def fxmvstop(tradeid,stop,flag_pips=False, demo=False):
  pips_arg = '--pips' if flag_pips else ''
  demo_arg = '--demo' if demo else '--real'
  cli_args = [CLI_FXMVSTOP_PROG_NAME, '-tid', tradeid, '-x', stop, demo_arg]
  if pips_arg != '':
    cli_args.append(pips_arg)
  subprocess.run(cli_args)

def ids(instrument, timeframe,use_full=False,use_fresh=True):
  use_fresh_arg = '-old' if not use_fresh else '--fresh'
  use_full_arg = '--full' if use_full else '-new'
  subprocess.run([IDSCLI_PROG_NAME, '-i', instrument, '-t', timeframe, use_full_arg, use_fresh_arg])

"""
fxmvstopgator -tid 68773276  --demo -i AUD/NZD -t H4 --lips
"""


from jgtpy.jgtapyhelper import select_value_in_currentbar,select_value_in_lastcompletedbar
#@STCGoal Move EXIT Stop On FDB Signal

#@STCGoal RM Order if Stop was Hit


def fxmvstopgator(i,t,tradeid,lips=True,teeth=False,jaw=False,demo=False):
  
  #First update the IDS
  try:
    ids(i,t,use_fresh=True,use_full=False)
  except:
    print("IDS failed")
  #read the last bar from the IDS csv
  from jgtpy import jgtapyhelper as th
  df=th.read_ids(i,t)
  #get the last bar
  last_bar=df.iloc[-1]
  choosen_line="_"
  #get the stop from choosen indicator line (by flag)
  if not teeth and not jaw:
    lips=True # DEFAULT
  if lips:
    stop=str(last_bar[LIPS])
    choosen_line="lips"
  elif teeth:
    stop=str(last_bar[TEETH])
    choosen_line="teeth"
  elif jaw:
    stop=str(last_bar[JAW])
    choosen_line="jaw"
  else:
    raise ValueError("No indicator line selected")
  print(f"Stop made of choosen line ({choosen_line}):: ",stop)
  #Then move the stop
  fxmvstop(tradeid,stop,demo=demo)
  
def tide(instrument, timeframe, buysell):
  raise DeprecationWarning("tide is deprecated. ")
  #subprocess.run(['tide', instrument, timeframe, buysell])

def pds(instrument, timeframe,use_full=True):
  use_full_arg = '--full' if use_full else ''
  subprocess.run([PDSCLI_PROG_NAME, '-i', instrument, '-t', timeframe, use_full_arg])

def cds(instrument, timeframe, use_fresh=False,use_full=True):
  use_full_arg = '--full' if use_full else ''
  old_or_fresh = '-old' if not use_fresh else '--fresh'
  subprocess.run([CDSCLI_PROG_NAME, '-i', instrument, '-t', timeframe,use_full_arg, old_or_fresh])

def ocds(instrument, timeframe,use_full=True):
  use_full_arg = '--full' if use_full else ''
  subprocess.run([JGTCLI_PROG_NAME, '-i', instrument, '-t', timeframe, use_full_arg, '-old'])

  
def ttf(instrument, timeframe,pn="ttf",use_fresh=False,use_full=True):
  use_full_arg = '--full' if use_full else ''
  use_fresh_arg = '-old' if not use_fresh else '--fresh'
  
  ttf_args = [TTFCLI_PROG_NAME, '-i', instrument, '-t', timeframe, use_full_arg, use_fresh_arg, '-pn', pn]
  print("TTF is being ran by jgtapp with args: ", ttf_args)
  subprocess.run(ttf_args)


def mlf(instrument, timeframe,pn="ttf",total_lagging_periods=5,use_fresh=False,use_full=True):
  use_full_arg = '--full' if use_full else ''
  use_fresh_arg = '-old' if not use_fresh else '--fresh'
  mlf_args = [MLFCLI_PROG_NAME, '-i', instrument, '-t', timeframe, use_full_arg, use_fresh_arg, '-pn', pn,'--total_lagging_periods',total_lagging_periods]
  subprocess.run(mlf_args)

  

def mx(instrument, timeframe, use_fresh=False):
  old_or_fresh = '-old' if not use_fresh else '--fresh'
  subprocess.run([MXCLI_PROG_NAME, '-i', instrument, '-t', timeframe, old_or_fresh])

def ttfmxwf(instrument, use_fresh=False):
  for t in ["M1", "W1", "D1", "H4"]:
    print(f"Processing {instrument} for timeframe {t}")
    print("  CDS....")
    cds(instrument, t, use_fresh)
    if t != "M1":
      print("  TTF....")
      ttf(instrument, t)
    if t != "M1" and t != "W1":
      print("  MX....")
      mx(instrument, t)
      
def ttfwf(instrument, use_fresh=False):
  from jgtutils import jgtcommon
  _settings = jgtcommon.get_settings()
  #ttf2run
  if hasattr(_settings, 'ttf2run'):
    ttf2run=_settings.ttf2run
  else:
    from jgtml.mlconstants import TTF2RUN_DEFAULT
    ttf2run=TTF2RUN_DEFAULT
  for pn in ttf2run:
      
    for t in ["M1", "W1", "D1", "H4"]:
      print(f"Processing {instrument} for timeframe {t}")
      print("  CDS....")
      cds(instrument, t, use_fresh)
      if t != "M1":
        print("  TTF....")
        ttf(instrument, t,pn=pn)
        
def main():
  parser = argparse.ArgumentParser(description="CLI equivalent of bash functions")
  subparsers = parser.add_subparsers(dest='command')
  
  #fxaddorder
  parser_fxaddorder = subparsers.add_parser('fxaddorder', help='Add an order')
  parser_fxaddorder.add_argument('-i','--instrument', help='Instrument')
  parser_fxaddorder.add_argument('-n','--lots', help='Lots')
  parser_fxaddorder.add_argument('-r','--rate', help='Rate')
  parser_fxaddorder.add_argument('-d','--buysell', help='Buy or Sell')
  parser_fxaddorder.add_argument('-x','--stop', help='Stop')
  parser_fxaddorder.add_argument('--demo', action='store_true', help='Use the demo account')
  parser_fxaddorder.add_argument('--pips', action='store_true', help='Use pips')
  
  #fxrmorder
  parser_fxrmorder = subparsers.add_parser('fxrmorder', help='Remove an order')
  parser_fxrmorder.add_argument('-id','--orderid', help='Order ID')
  parser_fxrmorder.add_argument('--demo', action='store_true', help='Use the demo account')
  
  #fxrmtrade
  parser_fxrmtrade = subparsers.add_parser('fxrmtrade', help='Remove a trade')
  parser_fxrmtrade.add_argument('-tid','--tradeid', help='Trade ID')
  parser_fxrmtrade.add_argument('--demo', action='store_true', help='Use the demo account')
  
  #fxtr
  parser_fxtr = subparsers.add_parser('fxtr', help='Get trade details')
  parser_fxtr.add_argument('-tid','--tradeid', help='Trade ID')
  parser_fxtr.add_argument('-id','--orderid', help='Order ID')
  parser_fxtr.add_argument('--demo', action='store_true', help='Use the demo account')
  parser_fxtr.add_argument('--nosave', action='store_true', help='Dont Save the trade details')
  
  #fxmvstop
  parser_fxmvstop = subparsers.add_parser('fxmvstop', help='Move stop')
  parser_fxmvstop.add_argument('-tid','--tradeid', help='Trade ID')
  parser_fxmvstop.add_argument('-x','--stop', help='Stop')
  parser_fxmvstop.add_argument('--pips', action='store_true', help='Use pips')
  parser_fxmvstop.add_argument('--demo', action='store_true', help='Use the demo account')
  
  #ids
  parser_ids = subparsers.add_parser('ids', help='Refresh the IDS')
  parser_ids.add_argument('-i','--instrument', help='Instrument')
  parser_ids.add_argument('-t','--timeframe', help='Timeframe')
  parser_ids.add_argument('--full', action='store_true', help='Use the full data')
  parser_ids.add_argument('--fresh', action='store_true', help='Use the fresh data')
  
  #fxmvstopgator
  parser_fxmvstopgator = subparsers.add_parser('fxmvstopgator', help='Move stop using gator')
  parser_fxmvstopgator.add_argument('-i','--instrument', help='Instrument')
  parser_fxmvstopgator.add_argument('-t','--timeframe', help='Timeframe')
  parser_fxmvstopgator.add_argument('-tid','--tradeid', help='Trade ID')
  parser_fxmvstopgator.add_argument('--lips', action='store_true', help='Use lips')
  parser_fxmvstopgator.add_argument('--teeth', action='store_true', help='Use teeth')
  parser_fxmvstopgator.add_argument('--jaw', action='store_true', help='Use jaw')
  parser_fxmvstopgator.add_argument('--demo', action='store_true', help='Use the demo account')
  

  parser_tidealligator = subparsers.add_parser('tide', help='Run the pto tidealligator')
  parser_tidealligator.add_argument('-i','--instrument', help='Instrument')
  parser_tidealligator.add_argument('-t','--timeframe', help='Timeframe')
  parser_tidealligator.add_argument('buysell', help='Buy or Sell')

  parser_prep_pds_01 = subparsers.add_parser('pds', help='Refresh the PDS full for an instrument and timeframe')
  parser_prep_pds_01.add_argument('-i','--instrument', help='Instrument')
  parser_prep_pds_01.add_argument('-t','--timeframe', help='Timeframe')
  parser_prep_pds_01.add_argument('--full', action='store_true', help='Use the full data')

  parser_prep_cds_05 = subparsers.add_parser('cds', help='Refresh the CDS')
  parser_prep_cds_05.add_argument('-i','--instrument', help='Instrument')
  parser_prep_cds_05.add_argument('-t','--timeframe', help='Timeframe')
  #--fresh flag to use the fresh data
  parser_prep_cds_05.add_argument('-new','--fresh', action='store_true', help='Use the fresh data')
  parser_prep_cds_05.add_argument('--full', action='store_true', help='Use the full data')

  parser_prep_cds_06_old = subparsers.add_parser('ocds', help='Refresh the CDS from old PDS')
  parser_prep_cds_06_old.add_argument('-i','--instrument', help='Instrument')
  parser_prep_cds_06_old.add_argument('-t','--timeframe', help='Timeframe')
  parser_prep_cds_06_old.add_argument('--full', action='store_true', help='Use the full data')
  
  
  parser_prep_ttf_10 = subparsers.add_parser('ttf', help='Refresh the TTF for an instrument and timeframe')
  parser_prep_ttf_10.add_argument('-i','--instrument', help='Instrument symbol')
  parser_prep_ttf_10.add_argument('-t','--timeframe', help='Timeframe')
  parser_prep_ttf_10.add_argument('-pn','--patternname', help='Pattern Name')
  parser_prep_ttf_10.add_argument('-new','--fresh', action='store_true', help='Use the fresh data')
  parser_prep_ttf_10.add_argument('--full', action='store_true', help='Use the full data')


  
  parser_prep_mlf_22 = subparsers.add_parser('mlf', help='Refresh the MLF for an instrument and timeframe')
  parser_prep_mlf_22.add_argument('-i','--instrument', help='Instrument symbol')
  parser_prep_mlf_22.add_argument('-t','--timeframe', help='Timeframe')
  parser_prep_mlf_22.add_argument('-pn','--patternname', help='Pattern Name')
  parser_prep_mlf_22.add_argument('-tlp','--total_lagging_periods', help='Total Lagging Periods')
  parser_prep_mlf_22.add_argument('-new','--fresh', action='store_true', help='Use the fresh data')
  parser_prep_mlf_22.add_argument('--full', action='store_true', help='Use the full data')
  
  
  #ttfmxwf
  parser_post_ttfmxwf_14 = subparsers.add_parser('ttfmxwf', help='Refresh the TTF, MX and CDS for an instrument')
  parser_post_ttfmxwf_14.add_argument('-i','--instrument', help='Instrument symbol')
  parser_post_ttfmxwf_14.add_argument('-new','--fresh', action='store_true', help='Use the fresh data')

  parser_post_mx_15 = subparsers.add_parser('mx', help='Refresh the MX (using the TTF) for an instrument and timeframe')
  parser_post_mx_15.add_argument('-i','--instrument', help='Instrument symbol')
  parser_post_mx_15.add_argument('-t','--timeframe', help='Timeframe')

  parser_wf_ttf_prep_19 = subparsers.add_parser('ttfwf', help='Refresh TTF preparation for an instrument')
  parser_wf_ttf_prep_19.add_argument('-i','--instrument', help='Instrument symbol')
  parser_wf_ttf_prep_19.add_argument('-new','--fresh', action='store_true', help='Use the fresh data')
  parser_wf_ttf_prep_19.add_argument('--full', action='store_true', help='Use the full data')

  args = parser.parse_args()
  #if no arguments are passed, print help
  if not vars(args).get('command'):
    parser.print_help()
    parser.exit()

  if args.command == 'tide':
    tide(args.instrument, args.timeframe, args.buysell)
  elif args.command == 'fxaddorder':
    fxaddorder(args.instrument, args.lots, args.rate, args.buysell, args.stop, args.demo,args.pips)
  elif args.command == 'fxrmorder':
    fxrmorder(args.orderid, args.demo)
  elif args.command == 'fxrmtrade':
    fxrmtrade(args.tradeid, args.demo)
  elif args.command == 'fxtr':
    fxtr(args.tradeid,args.orderid,args.demo,not args.nosave)
  elif args.command == 'fxmvstop':
    fxmvstop(args.tradeid, args.stop, args.pips, args.demo)
  elif args.command == 'ids':
    ids(args.instrument, args.timeframe,args.full,args.fresh)
  elif args.command == 'fxmvstopgator':
    lips_value = True if not args.lips and not args.teeth and not args.jaw else False
    fxmvstopgator(args.instrument, args.timeframe, args.tradeid, lips_value,args.teeth,args.jaw,args.demo)
  elif args.command == 'pds':
    pds(args.instrument, args.timeframe,)
  elif args.command == 'cds':
    cds(args.instrument, args.timeframe, args.fresh,args.full)
  elif args.command == 'ocds':
    ocds(args.instrument, args.timeframe)
  elif args.command == 'ttf':
    ttf(args.instrument, args.timeframe,args.patternname,args.columns_list_from_higher_tf,args.fresh,args.full)
  elif args.command == 'mlf':
    mlf(args.instrument, args.timeframe,args.patternname,args.total_lagging_periods,args.fresh,args.full)
  elif args.command == 'ttfmxwf':
    ttfmxwf(args.instrument, args.fresh)
  elif args.command == 'mx':
    mx(args.instrument, args.timeframe)
  elif args.command == 'ttfwf':
    ttfwf(args.instrument, args.fresh,args.full)

if __name__ == "__main__":
  main()



#!/usr/bin/env python

import sys
import os

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

# import .

from mlcliconstants import MXCLI_DESCRIPTION, MXCLI_EPILOG, MXCLI_PROG_NAME

from jgtutils import (
    jgtcommon as jgtcommon
)
import argparse

import pandas as pd

#import jgtml as jml
import  jtc

import jplt

import tlid



import pandas as pd


def parse_args():
    parser = jgtcommon.new_parser(MXCLI_DESCRIPTION,MXCLI_EPILOG,MXCLI_PROG_NAME)
    # jgtfxcommon.add_main_arguments(parser)
    jgtcommon.add_instrument_timeframe_arguments(parser)
    
    jgtcommon.add_tlid_range_argument(parser)
    
    jgtcommon.add_verbose_argument(parser)
    
    jgtcommon.add_use_full_argument(parser)
    #add_use_fresh_argument
    jgtcommon.add_use_fresh_argument(parser)
    
    jgtcommon.add_keepbidask_argument(parser)

    jgtcommon.add_ids_mfi_argument(parser)
    jgtcommon.add_ids_gator_oscillator_argument(parser)
    jgtcommon.add_ids_balligator_argument(parser)
    jgtcommon.add_ids_talligator_argument(parser)
    jgtcommon.add_ids_fractal_largest_period_argument(parser)
    jgtcommon.add_dropna_volume_argument(parser)
    
    #add argument to regenerate the cds
    parser.add_argument('-rcds', '--regenerate_cds', action='store_true', help='Regenerate the CDS')

    jgtcommon.add_selected_columns_arguments(parser)
    #parser.add_argument('-sc', '--selected-columns', nargs='+', help='List of selected columns to keep', default=['High','Low','ao','ac','jaw','teeth','lips','fh','fl','fdbb','fdbs','zlcb','zlcs','target','vaosc','vaobc'])

    # dont drop calculated columns
    parser.add_argument('-ddcc', '--dont_drop_calc_col', action='store_true', help='Dont drop calculated columns')
    
    parser=jgtcommon.add_patterns_arguments(parser)
    #parser.add_argument("-pn", "--patternname", help="Pattern Name", default="ttf")
  
    # jgtcommon.add_cds_argument(parser)
    args = parser.parse_args()
    return args


def main():
    
    args = parse_args()
    regenerate_cds=args.regenerate_cds

    verbose_level = args.verbose
    quiet = False
    if verbose_level == 0:
        quiet = True

    instrument = args.instrument
    timeframe = args.timeframe
    
    keep_bid_ask = True
    if args.rmbidask:
        keep_bid_ask = False
        
    full = True
    if args.notfull:
        full = False
        print_quiet(quiet, "USING FULL MODE TURNED OFF")

    fresh = True
    if args.notfresh:
        fresh = False
        print_quiet(quiet, "USING FRESH MODE TURNED OFF")

    date_from = None
    date_to = None
    tlid_range = None
    if args.tlidrange:
        # @STCGoal Get range prices from cache or request new
        tlid_range = args.tlidrange
        print("FUTURE Support for tlid range")
        
        print("-----------Stay tune -------- Quitting for now")
        return


    dont_drop_calc_col=True if args.dont_drop_calc_col else False

    

    if verbose_level > 1:
        if date_from:
            print("Date from : " + str(date_from))
        if date_to:
            print("Date to : " + str(date_to))

    try:

        print_quiet(quiet, "Getting for : " + instrument + "_" + timeframe)
        instruments = instrument.split(",")
        timeframes = timeframe.split(",")

        for instrument in instruments:
            for timeframe in timeframes:
                print("-------JTC Processing : " + instrument + "_" + timeframe)

                #selected_columns_to_keep  =['High','Low','ao','ac','jaw','teeth','lips','fh','fl','fdbb','fdbs','zlcb','zlcs','target','vaos','vaob']
                selected_columns_to_keep = args.selected_columns
   
                #selected_columns_to_keep=['Volume','High','Low','ao','ac','jaw','teeth','lips','fh','fl','fdbb','fdbs','aocolor','accolor','zcol','sz','bz','acs','acb','ss','sb','price_peak_above','price_peak_bellow','ao_peak_above','ao_peak_bellow']
                if full:
                    #Full column
                    selected_columns_to_keep = None
        
                r,s1,s2= jtc.pto_target_calculation( #@STCIssue Exception: 'NoneType' object has no attribute 'append';jgtmlcli.py", line 120;tc.py", line 189, in pto_target_calculation;line 337, in _pov_target_calculation_n_output240223selected_columns_to_keep.append(VECTOR_AO_FDB_COUNT) AttributeError: 'NoneType' object has no attribute 'append'
                    instrument,
                    timeframe,
                    pto_vec_fdb_ao_vector_window_flag=True,
                    drop_calc_col=dont_drop_calc_col,
                    selected_columns_to_keep=selected_columns_to_keep,
                    save_outputs=True,
                    keep_bid_ask=keep_bid_ask,
                    use_fresh=fresh,
                    regenerate_cds=regenerate_cds,
                    gator_oscillator_flag=args.gator_oscillator_flag,
                    balligator_flag=args.balligator_flag,largest_fractal_period=args.largest_fractal_period,
                    mfi_flag=args.mfi_flag,
                    balligator_period_jaws=args.balligator_period_jaws,
                    pn=args.patternname
                    
                    )

    except Exception as e:
        jgtcommon.print_exception(e)

    # try:
    #    jgtpy.off()
    # except Exception as e:
    #    jgtfxcommon.print_exception(e)


# if __name__ == "__main__":
#     main()

# print("")
# #input("Done! Press enter key to exit\n")


def createMX_for_main(
    instrument,
    timeframe,
    quiet,
    verbose_level=0,
    tlid_range=None,
    use_full=False,
):
    print("----TODO:  createMX_for_main")

def print_quiet(quiet, content):
    if not quiet:
        print(content)


if __name__ == "__main__":
    main()






import tlid
from jgtutils import iprops
from jgtutils.jgtconstants import \
  HIGH,LOW,FDB,ASKHIGH,ASKLOW,BIDHIGH,BIDLOW,JAW,TEETH,LIPS,BJAW,BTEETH,BLIPS

#@STCGoal Standardize the Signal Columns
from jgtml.mlconstants import (
  NORMAL_MOUTH_IS_OPEN_COLNAME,
  CURRENT_BAR_IS_OUT_OF_NORMAL_MOUTH_COLNAME,
  CURRENT_BAR_IS_IN_BIG_TEETH_COLNAME,
  BIG_MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_BIG_LIPS_COLNAME,
  MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_BIG_TEETH_COLNAME,
  CURRENT_BAR_IS_IN_TIDE_TEETH_COLNAME,
  TIDE_MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_TIDE_LIPS_COLNAME,
  MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_TIDE_TEETH_COLNAME
)

from jgtutils.iprops import \
  get_pips

def get_entry_stop_rate_ticked(i,bs,entry_rate,stop_rate,tick_shift=1,rouding_add = 2,t=None):
  pips=get_pips(i)
  tick_size=pips/10
  
  decimal_places = len(str(pips).split('.')[1]) + rouding_add if '.' in str(pips) else 1
  if bs=="B":
    entry_rate+=tick_size*tick_shift
    stop_rate-=tick_size*tick_shift
  else:
    if bs=="S":
      entry_rate-=tick_size*tick_shift
      stop_rate+=tick_size*tick_shift
  entry_rate = round(entry_rate, decimal_places)
  stop_rate = round(stop_rate, decimal_places)
  return entry_rate,stop_rate
  

def valid_gator(last_bar_completed,current_bar,bs):
  last_bar_mouth_is_open_and_price_is_out = is_mouth_open_and_bar_out_of_it(last_bar_completed,bs)
  cur_bar_mouth_is_open_and_price_is_out = is_mouth_open_and_bar_out_of_it(current_bar,bs)
  return \
    last_bar_mouth_is_open_and_price_is_out \
      and \
        cur_bar_mouth_is_open_and_price_is_out



def is_mouth_open_and_bar_out_of_it(bar,bs)->bool:
  return is_bar_out_of_mouth(bar,bs) and is_mouth_open(bar,bs)


def is_bar_out_of_mouth(bar,bs)->bool:
  mouth_open_reverse = is_mouth_open(bar,"B") if bs=="S" else is_mouth_open(bar,"S")
  if bs=="B":
    return bar[HIGH] < bar[LIPS] \
      and \
        not mouth_open_reverse
  if bs=="S":
    return bar[LOW] > bar[LIPS] \
      and \
        not mouth_open_reverse
  
def is_mouth_open(bar,bs)->bool:
  if bs=="B":
    return  bar[LIPS] < bar[TEETH] and bar[TEETH] < bar[JAW] and  bar[LIPS] < bar[JAW]
  if bs=="S":
    return  bar[LIPS] > bar[TEETH] and bar[TEETH] > bar[JAW]and  bar[LIPS] > bar[JAW]
  
def is_big_mouth_open(bar,bs)->bool:
  if bs=="B":
    return  bar[BLIPS] < bar[BTEETH] and bar[BTEETH] < bar[BJAW] and  bar[BLIPS] < bar[BJAW]
  if bs=="S":
    return  bar[BLIPS] > bar[BTEETH] and bar[BTEETH] > bar[BJAW]and  bar[BLIPS] > bar[BJAW]

def is_fdbsignal_crossed_t(bar,bs,tcol):
  if bs=="B":
    return bar[HIGH] < bar[tcol]
  if bs=="S":
    return bar[LOW] > bar[tcol]

def is_fdbsignal_in_big_mouth(bar,bs):
  return is_fdbsignal_crossed_t(bar,bs,BLIPS)
  
def is_fdbsignal_in_big_mouth_teeth(bar,bs):
  return is_fdbsignal_crossed_t(bar,bs,BLIPS)

def create_fdb_entry_order(i,signal_bar,current_bar,lots=1,tick_shift=2,quiet=True,valid_gator_mouth_open_in_mouth=False,valid_sig_out_mouth=True,t=None):
  had_valid_signal=False
  current_bar_broke_signal=True
  
  tlid_id = tlid.get_seconds()
  
  if signal_bar[FDB]==1:
    
    askhigh = signal_bar[ASKHIGH]
    bidlow = signal_bar[BIDLOW]
    
    entry_rate,stop_rate=get_entry_stop_rate_ticked(i,"B",askhigh,bidlow,tick_shift=tick_shift,t=t)

    buysell="B"
    had_valid_signal=True
    #check of the current_bar make the signal invalid by being out of range (means the entry_rate is already hit or it passed the stop_rate (making it invalid))
    cur_askhigh = current_bar[ASKHIGH]
    cur_bidlow = current_bar[BIDLOW]
    
    if cur_askhigh<=entry_rate and cur_bidlow>=stop_rate :
      current_bar_broke_signal=False
    if not quiet:print(f"cur_askhigh:{cur_askhigh} entry_rate:{entry_rate} cur_bidlow:{cur_bidlow} stop_rate:{stop_rate}")
  if signal_bar[FDB]==-1:
    #print(f"# Sell Signal on {i}")
    
    bidlow = signal_bar[BIDLOW]
    askhigh = signal_bar[ASKHIGH]
    
    entry_rate,stop_rate=get_entry_stop_rate_ticked(i,"S",bidlow,askhigh,tick_shift=tick_shift,t=t)

    buysell="S"
    had_valid_signal=True
    
    cur_bidlow = current_bar[BIDLOW]
    cur_askhigh = current_bar[ASKHIGH]
    
    if cur_bidlow>=entry_rate and cur_askhigh<=stop_rate:
      current_bar_broke_signal=False
    
    if not quiet:print(f"cur_bidlow:{cur_bidlow} entry_rate:{entry_rate} cur_askhigh:{cur_askhigh} stop_rate:{stop_rate}")
  
  if  current_bar_broke_signal and had_valid_signal:
    print(f"## Current Bar Broke the Signal {i}")
    return None
  
  if not had_valid_signal:
    return None
  
  if valid_gator_mouth_open_in_mouth \
    and \
      not valid_gator(signal_bar,current_bar,buysell):
    print(f"## Invalid Gator {i} valid_gator_mouth_open_in_mouth")
    return None
  
  if valid_sig_out_mouth \
    and \
      not is_bar_out_of_mouth(current_bar,buysell):
    print(f"## Invalid Gator {i} not valid_sig_out_mouth")
    return None
  
  output_script = generate_entry_order_script(lots, entry_rate, stop_rate, i, buysell,tlid_id=tlid_id,t=t)
  
  o = build_order_result_object(lots, entry_rate, stop_rate, buysell, tlid_id, output_script,i,t)
  
  return o

def build_order_result_object(lots, entry_rate, stop_rate, buysell, tlid_id, output_script,i,t):
    o={}
    o["sh"]=output_script
    o["entry"]=entry_rate
    o["stop"]=stop_rate
    o["bs"]=buysell
    o["lots"]=lots
    o["tlid_id"]=tlid_id
    o["i"]=i
    o["t"]=t
    return o


def generate_entry_order_script(lots, entry_rate, stop_rate, instrument, buysell,tlid_id=None,t=None):
    timeframe=t if t is not None else "_"
    if tlid_id is None:
      tlid_id = tlid.get_seconds()
    output_script=f"""
```sh
### --- COPY FROM HERE ---
# Entry Order for {instrument} {timeframe} {buysell}
demo_arg=" --demo"
tlid_id={tlid_id}
jgtnewsession $tlid_id {instrument} {timeframe} {entry_rate} {stop_rate} {buysell} {lots} $demo_arg
### ---- COPY TO HERE ---
```
"""
    
    return output_script

def generate_entry_order_script_pto1(lots, entry_rate, stop_rate, instrument, buysell):
    output_script=f"""
```sh
entry_rate={entry_rate};stop_rate={stop_rate};instrument={instrument};buysell={buysell};lots={lots}
s=CreateEntryOrderPtoAddStop.py
python $s $real_fx_cli_base_args -lots $lots -r $entry_rate -d $buysell -i $instrument -stop $stop_rate  | tee __output.txt  && \
OrderID=$(cat __output.txt| grep -o 'OrderID=[0-9]*' | cut -d '=' -f2) && \
echo "OrderID: $OrderID"
```
"""
    
    return output_script


def get_iprop(i):
  return iprops.get_iprop(i)


import os

class JGTML240615Config:
    def __init__(self):
        self.jgtpy_data_full_var_name = "JGTPY_DATA_FULL"
        self.result_drop_base_override = None
        self.source_dataset_archival_path_override = None
        self.data_dir_override = None
        self.archive_used_dataset = True
        self.use_ttf_default = True
        # Additional static configurations can be added here

class JGTML240615Request:
    def __init__(self, instrument=None, timeframe=None, buysell='S', regenerate_cds=True, use_fresh=True, quiet=False):
        self.instrument = instrument
        self.timeframe = timeframe
        self.buysell = buysell
        self.regenerate_cds = regenerate_cds
        self.use_fresh = use_fresh
        self.quiet = quiet
        # Initialize from environment variables if not explicitly set
        self.buysell = os.getenv("bs", buysell)

    @staticmethod
    def from_args(args):
        """
        Create a JGTML240615Request instance from an argparse.Namespace.
        This allows for easy integration with command-line arguments.
        """
        return JGTML240615Request(
            instrument=args.instrument,
            timeframe=args.timeframe,
            buysell=args.buysell,
            regenerate_cds=not args.dont_regenerate_cds,
            use_fresh=not args.use_old,
            quiet=args.quiet
        )


import pandas as pd

from mlconstants import MFI_DEFAULT_COLNAME
from jgtutils.colconverthelper import mfi_str_to_id,mfi_signal_to_str,get_mfi_features_column_list_by_timeframe


def column_mfi_str_in_dataframe_to_id(df:pd.DataFrame,t:str,inplace=False,mfi_colname=""):
    """
    Convert the MFI string columns in the dataframe to their corresponding MFI ID.
    
    Parameters:
    df (pd.DataFrame): The dataframe to convert the MFI string columns to their corresponding MFI ID.
    t (str): The timeframe to get the MFI features columns for.
    mfi_colname (str): The name of the MFI column to use. Default is MFI_VAL (plan to upgrade to MFI_SIGNAL)
    
    Returns:
    pd.DataFrame: The dataframe with the MFI string columns converted to their corresponding MFI ID.
    """
    if mfi_colname=="":
        mfi_colname=MFI_DEFAULT_COLNAME
    
    if not inplace:
        df = df.copy()
    
    mfi_str_selected_columns=get_mfi_features_column_list_by_timeframe(t,mfi_colname)
    
    for col_name in mfi_str_selected_columns:
        #check if the column exists in the dataframe
        if col_name not in df.columns:
            continue
        df[col_name] = df[col_name].apply(lambda x: int(mfi_str_to_id(x)))
    return df

def column_mfi_str_back_to_str_in_dataframe(df:pd.DataFrame,t:str,inplace=False,mfi_colname=""):
    if not inplace:
        df = df.copy()    
    mfi_str_selected_columns=get_mfi_features_column_list_by_timeframe(t,mfi_colname)
    for col_name in mfi_str_selected_columns:
        #check if the column exists in the dataframe
        if col_name not in df.columns:
            continue
        df[col_name] = df[col_name].apply(lambda x: mfi_signal_to_str(x)).copy()
    return df


import json
import os

def generate_markdown_from_json_file(json_filepath):
    with open(json_filepath, 'r') as file:
        data = json.load(file)
    
    signalbar = data['signalbar']
    currentbar = data['currentbar']
    
    markdown_output = f"""
# Trade Signal Details

## Trade Information
- **Entry Price:** {data['entry']}
- **Stop Loss:** {data['stop']}
- **Buy/Sell:** {data['bs']}
- **Lots:** {data['lots']}
- **Tlid ID:** {data['tlid_id']}
- **Instrument:** {data['i']}
- **Timeframe:** {data['t']}

![](signal.png)

[../charts](../charts)
[M1](../charts/M1.png)-[W1](../charts/W1.png)-[D1](../charts/D1.png)-[H4](../charts/H4.png)-[H1](../charts/H1.png)-[m15](../charts/m15.png)-[m5](../charts/m5.png)

"""



    markdown_output += """

----

## Utilities

* [entry.sh](../.jgt/entry.sh)
* [stop.sh](../.jgt/stop.sh)
* [watch.sh](../.jgt/watch.sh)
* [status.sh](../.jgt/status.sh)
* [update.sh](../.jgt/update.sh)
* [env.sh](../.jgt/env.sh)

### --@STCIssue Future Enhancements

* CLI Commands to run, not hard coded scripts
* Example : _fxtrupdate, _jgtsession_mksg, _jgtsession_vswsopen, _jgtsession_mkads_ctx_timeframe, _jgtsession_mkads_all_timeframes

"""



    markdown_output += f"""

## Signal Bar Data
| Metric           | Value         |
|------------------|---------------|
"""
    for key, value in signalbar.items():
        markdown_output += f"| {key} | {value} |\n"

    markdown_output += """
## Current Bar Data
| Metric           | Value         |
|------------------|---------------|
"""
    for key, value in currentbar.items():
        markdown_output += f"| {key} | {value} |\n"


    return markdown_output

def serialize_signal_to_markdown_file_from_json_file(json_filepath,quiet=True):
    markdown_output = generate_markdown_from_json_file(json_filepath)
    markdown_filepath = json_filepath.replace('.json', '.md')
    try:  
      with open(markdown_filepath, 'w') as file:
          file.write(markdown_output)
          if not quiet:print(f">Markdown file saved to: {markdown_filepath}")
      return markdown_filepath
    except Exception as e:
      print(f"Error saving markdown file to {markdown_filepath} {e}")
      return None
  
def serialize_signal_to_json_file(i,t,o,signal_bar,current_bar,quiet=True,signal_dir="data/signals",ext="json",indent=2):
      
      tlid_id=o["tlid_id"]
      o["i"]=i
      o["t"]=t
      o["signalbar"]=signal_bar.to_dict()
      o["currentbar"]=current_bar.to_dict()
      
      signal_fn = create_signal_filename(i,t, tlid_id, ext)
      signal_savepath =os.path.join(signal_dir,signal_fn)
      try:      
        os.makedirs(signal_dir,exist_ok=True)
        json.dump(o,open(signal_savepath,"w"),indent=indent)
        if not quiet:print(f">Signal saved to :{signal_savepath}")
        return signal_savepath
      except Exception as e:
        print(f"Error saving signal to {signal_savepath} {e}")
        return None

def create_signal_savepath(i,t, tlid_id, ext, signal_dir="data/signals"):
    signal_fn = create_signal_filename(i,t, tlid_id, ext)
    signal_savepath =os.path.join(signal_dir,signal_fn)
    return signal_savepath

def create_signal_filename(i,t, tlid_id, ext):
    ifn=i.replace("/","-")
    filename = f"{ifn}_{t}_{tlid_id}.{ext}"
    return filename


#%% Imports
import pandas as pd
import os
#import jgtml as jml
from jgtml import  jtc

from jgtml import jplt

import tlid
tlid_tag = tlid.get_minutes()


crop_end_dt=None;crop_start_dt=None

I_raw = os.getenv('I')
T_raw = os.getenv('T')

if I_raw is None or T_raw is None:
    raise ValueError("Environment variables 'I' and 'T' must be set.")

instruments = I_raw.split(',')
timeframes = T_raw.split(',')



print("Processing", I_raw, T_raw)
for i in instruments:
    for t in timeframes:
        print("Processing POV:" , i, t)
        #Date,Volume,Open,High,Low,Close,Median,ao,ac,jaw,teeth,lips,fh,fl,fh3,fl3,fh5,fl5,fh8,fl8,fh13,fl13,fh21,fl21,fh34,fl34,fh55,fl55,fh89,fl89,fdbb,fdbs,fdb,aof,aofvalue,aoaz,aobz,zlc,zlcb,zlcs,aocolor,accolor,zcol,sz,bz,acs,acb,ss,sb,price_peak_above,price_peak_bellow,ao_peak_above,ao_peak_bellow,target,vector_ao_fdbs,vector_ao_fdbb
        # additional_columns_to_drop=['Volume','Open','High','Low','Close','Median','ao','ac','jaw','teeth','lips','fh','fl','fh3','fl3','fh5','fl5','fh8','fl8','fh13','fl13','fh21','fl21','fh34','fl34','fh55','fl55','fh89','fl89','fdbb','fdbs','fdb','aof','aofvalue','aoaz','aobz','zlc','zlcb','zlcs','aocolor','accolor','zcol','sz','bz','acs','acb','ss','sb','price_peak_above','price_peak_bellow','ao_peak_above','ao_peak_bellow','target','vector_ao_fdbs','vector_ao_fdbb']
        # additional_columns_to_drop=['Volume','Open','High','Low','Close','Median','ao','ac','jaw','teeth','lips','fh','fl','fh3','fl3','fh5','fl5','fh8','fl8','fh13','fl13','fh21','fl21','fh34','fl34','fh55','fl55','fh89','fl89','fdbb','fdbs','fdb','aof','aofvalue','aoaz','aobz','zlc','zlcb','zlcs','aocolor','accolor','zcol','sz','bz','acs','acb','ss','sb','price_peak_above','price_peak_bellow','ao_peak_above','ao_peak_bellow','target','vector_ao_fdbs','vector_ao_fdbb']
        selected_columns_to_keep  =['High','Low','ao','ac','jaw','teeth','lips','fh','fl','fdbb','fdbs','zlcb','zlcs','target','vector_ao_fdbs','vector_ao_fdbb']
        #selected_columns_to_keep=['Volume','High','Low','ao','ac','jaw','teeth','lips','fh','fl','fdbb','fdbs','aocolor','accolor','zcol','sz','bz','acs','acb','ss','sb','price_peak_above','price_peak_bellow','ao_peak_above','ao_peak_bellow']
        
        
        r,s1,s2= jtc.pto_target_calculation(i,t,pto_vec_fdb_ao_vector_window_flag=True,
                drop_calc_col=False,
                selected_columns_to_keep=selected_columns_to_keep)
        


